#pragma once

#include <Arduino.h>
#include <Esp.h>
#include <esp_err.h>

namespace UrsCPU {

   // Siehe auch EspClass in esp.h. Dort gibt es eine Reihe weiterer Funktionen zum Thema.

   class UrsEspClass : public EspClass
   {
   public:
      // Ruf den Grund fr das Zurcksetzen der CPU ab.
      // Die Enumeration RESET_REASON fr die Grnde findet man in <cpu>rom/rtc.h
      static int getResetReason(int cpuNumber);

      // Liefert die Bezeichnung des Reset-Grundes, z.B. POWERON_RESET.
      static String getResetReasonName(int reason);

      // Liefert die Beschreibung des Reset-Grundes, z.B. Vbat power on reset.
      static String getResetReasonDescription(int reason);

      // Registriert einen Shutdown-Handler
      // Mit dieser Funktion kann einen Handler registriert werden, der aufgerufen wird, 
      // bevor die Anwendung mit der Funktion softwareReset neu gestartet wird.
      // Mgliche Fehler (Return-Code, s. esp_err.h)
      //   - ESP_OK bei Erfolg.
      //    ESP_ERR_INVALID_STATE, wenn der Handler bereits registriert wurde.
      //   - ESP_ERR_NO_MEM, wenn keine Shutdown - Handler - Slots mehr verfgbar sind.
      static esp_err_t registerShutdownHandler(void (*shutdownHandler)(void));

      // Registrierung des Shutdown-Handlers aufheben
      // Mgliche Fehler (Return-Code, s. esp_err.h)
      //   - ESP_OK bei Erfolg.
      //   - ESP_ERR_INVALID_STATE, wenn der angegebene Handler noch nicht registriert wurde.
      static esp_err_t unregisterShutdownHandler(void (*shutdownHandler)(void));

      // Ruft die Gre des grten freien Speicherblocks ab, der fr DMA-Zugriff zugewiesen werden kann.
      static uint32_t getLargestFreeDmaHeapBlock();

      // Allokiert size Bytes nicht initialisierten Speicher, auf den per DMA zugegriffen werden kann.
      // Liefert einen Zeiger auf den allokierten Bereich oder NULL (nullptr), wenn eine Zuweisung nicht mglich war.
      // Um ein Speicherleck zu vermeiden, muss der zurckgegebene Zeiger mit free() oder dmaRealloc() freigegeben werden.
      static void* allocDmaHeapBlock(size_t size);
      static void* dmaMalloc(size_t size) { return allocDmaHeapBlock(size); }

      // Allokiert den angegebenen Speicherbereich neu.
      // Bereits vorhandene Zeiger in den Speicherbereich knnen dabei ungltig werden!
      static void* reallocDmaHeapBlock(void* ptr, size_t newSize);
      static void* dmaRealloc(void* ptr, size_t newSize) { return reallocDmaHeapBlock(ptr, newSize); }

      // Liefert true, wenn PSRAM vorhanden ist.
      static bool psramFound();

      // Allokiert size Bytes nicht initialisierten Speicher. Liefert einen Zeiger auf den allokierten Bereich oder NULL(nullptr),
      // wenn eine Zuweisung nicht mglich war.
      // Um ein Speicherleck zu vermeiden, muss der zurckgegebene Zeiger mit free() oder realloc() freigegeben werden.
      static void* psramMalloc(size_t size);

      // Allokiert den angegebenen Speicherbereich neu.
      // Bereits vorhandene Zeiger in den Speicherbereich knnen dabei ungltig werden!
      static void* psramRealloc(void* ptr, size_t newSize);


   };

   extern UrsEspClass UrsESP;
}